==> crow-2.11.0.tar.bz2: crow-2.11.0/crow/views.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef CROW_VIEWS_H
#define CROW_VIEWS_H

#include "util.h"
#include "value.h"
#include "containers.h"
#include "crow_impl.h"

namespace Crow {

typedef sigc::slot<Any, Property *> PropertyValueGet;
typedef sigc::slot<void, Property *, Any> PropertyValueSet;
typedef sigc::slot<void, Property *, PEntityView> PropertyViewConfigure;
typedef sigc::slot<PNode, Property *, int> PropertyElementInsert;
typedef sigc::slot<std::string, Property *, PNode> PropertyLabelGet;

enum PropertyRole { prNone, prScalar, prVector, prEntity };

class Property {

public:
	Property();

	Property(
		const std::string & name,
		PropertyRole role,
		TypeId type
	);

	const std::string * getName() { return &name; }
	PropertyRole getRole() { return role; }
	TypeId getType() { return type; }

	Any getValue();
	void setValue(Any value);
	void configureView(PEntityView view);
	PNode insertElement(int index);
	std::string getLabel(PNode node);

	Property & setValueGet(const PropertyValueGet & get) { valueGet= get; return *this; }
	Property & setValueSet(const PropertyValueSet & set) { valueSet= set; return *this; }
	Property & setViewConfigure(const PropertyViewConfigure & configure) { viewConfigure= configure; return *this; }
	Property & setElementInsert(const PropertyElementInsert & insert) { elementInsert= insert; return *this; }
	Property & setLabelGet(const PropertyLabelGet & get) { labelGet= get; return *this; }

	StateFlags getState() { return state; }
	StateFlags getNodeState() { return state & sfNodeStateMask; }
	Property & setFlag(StateFlags sf) { state |= sf; return *this; }
	Property & clearFlag(StateFlags sf) { state &= ~sf; return *this; }
	bool hasFlag(StateFlags sf) { return (state & sf) == sf; }
	bool isHidden() { return hasFlag(sfHidden); }
	bool isEditingAllowed() { return !hasFlag(sfNoEditing); }
	bool isReadOnly() { return hasFlag(sfReadOnly); }
	bool canEdit();

	EditorId getEditor() { return editor; }
	Property & setEditor(const std::string & ename);
	EditorId getElementEditor() { return elementEditor; }
	Property & setElementEditor(const std::string & ename);
	Any getDefault();
	Property & setDefault(Any dvalue);
	Any getInert();
	Property & setInert(Any value);
	Property & setInertDefault(Any value);

private:
	std::string name;
	PropertyRole role;
	TypeId type;
	PropertyValueGet valueGet;
	PropertyValueSet valueSet;
	PropertyViewConfigure viewConfigure;
	PropertyElementInsert elementInsert;
	PropertyLabelGet labelGet;
	StateFlags state;
	EditorId editor;
	EditorId elementEditor;
	Any defaultValue;
	Any inert;

};

class EntityView : public Object {

public:
	EntityView();

	void prepare(TypeId tid);
	void init(PNode node, Controller * controller, Any value=Any());
	void done();

	bool reconstruct();

	TypeId getType() { return type; }
	GType getGType();
	PNode getModelNode();

	Any getValue();

	bool isConfigured() { return configured; }
	void setConfigured();

	void update();
	void write();

	virtual bool isActiveWidget(Gtk::Widget * widget) { return true; }
	Property * findProperty(const std::string & pname);

protected:
	typedef std::list<Property> Properties;

	Properties * getProperties() { return &properties; }
	Controller * getController() { return controller; }
	Model * getModel();

	bool isEditing();
	bool isSeeking();
	bool isRoot();
	bool isInternal() { return internal; }

	void setValue(Any value);
	void clearValue();

	virtual Any createInstance();
	virtual void initInstance() { }
	virtual void doneInstance() { }

	void initDefaultValues();
	virtual void initDesign() { }

	Property & addProperty(
		const std::string & name,
		PropertyRole role,
		const std::string & type,
		Any dvalue=Any()
	);

	Property & addInertProperty(
		const std::string & name,
		PropertyRole role,
		const std::string & type,
		Any dvalue=Any()
	);

	Any getPropertyValueInert(Property * property);
	void setPropertyValueInert(Property * property, Any value);

	void moveBefore(const std::string & pname, const std::string & before);
	void touch(const std::string & str);
	void touch(PNode node, const std::string & str);
	Property * clearVectorProperty(const std::string & name);
	bool propertyModelVectorEmpty(const std::string & name);
	Any getPropertyModel(const std::string & name);

	template <class T> T getPropertyModel(const std::string & name, const T & defval) {
		Any value= getPropertyModel(name);
		return value ? *value->get<T>() : defval;
	}

private:
	TypeId type;
	PNode modelNode;
	Controller * controller;
	Any value;
	bool configured;
	bool internal;
	bool onceWritten;
	Properties properties;

};

class GlibObjectView : public EntityView {

public:
	GlibObjectView();

	Any getPropertyValue(Property * property);
	void setPropertyValue(Property * property, Any value);

	void setPropertyValueInertGet(Property * property, Any value);

	PNode insertObject(Property * property, int index);
	PNode insertEntity(Property * property, int index);
	PNode insertScalar(Property * property, int index, Any value);

	std::string findSignal(const std::string & name);

	PGlibObject getObject();
	template <class T> Glib::RefPtr<T> getObject() { return Glib::RefPtr<T>::cast_static(getObject()); }

protected:
	void doneInstance();

	void setReportDestruction(bool rd) { reportDestruction= rd; }

	Property & addProperty(
		const std::string & name,
		PropertyRole role,
		const std::string & type,
		Any dvalue=Any()
	);

	Property & addInertGetProperty(
		const std::string & name,
		PropertyRole role,
		const std::string & type,
		Any dvalue=Any()
	);

	PNode insertSignal(Property * property, int index);
	std::string getSignalLabel(Property * property, PNode node);

	Property & addSignalsProperty();

private:
	bool reportDestruction;

};

template <class T> class TCreateInstance : public virtual GlibObjectView {

public:
	Glib::RefPtr<T> getObject() { return GlibObjectView::getObject<T>(); }

protected:
	Any createInstance() { return CAny::createObject(getType(), NewRefPtr<T>()); }

};

struct FreePlace {
	Place place;
	Rectangle area;
};

typedef std::vector<FreePlace> FreePlaces;

class GtkWidgetView : public virtual GlibObjectView {

public:
	GtkWidgetView();

	virtual bool findFreePlace(const Point & p, FreePlace * fplace) { return false; }
	virtual FreePlaces findFreePlaces(const Point & p, const Nodes & nodes, const Point & p0) { return FreePlaces(); }
	virtual PNode createChild(const Place & place, PNode widget);

	virtual bool isTopChild(PChild child) { return true; }
	virtual Rectangle getChildArea();
	bool isInteractive() { return interactive; }

	PGtkWidget getObject();

protected:
	void initInstance();
	Any getSizeRequest(Property * property);
	void setSizeRequest(Property * property, Any value);

	Glib::ustring getDesignLabel();

	void addReceivesDefaultProperty(bool defval);
	void addCanFocusProperty(bool defval);
	void addCanDefaultProperty(bool defval);
	void addAppPaintableProperty();
	void addEventProperties();

	Point transform(const Point & p, PGtkWidget widget);
	Rectangle getWidgetRect(PGtkWidget widget);
	void setInteractive(bool b) { interactive= b; }

private:
	bool interactive;

};

class GtkContainerViewBase : public virtual GtkWidgetView {

public:
	GtkContainerViewBase();

};

class GtkContainerView : public GtkContainerViewBase {

public:
	GtkContainerView();

	bool findFreePlace(const Point & p, FreePlace * fplace);
	FreePlaces findFreePlaces(const Point & p, const Nodes & nodes, const Point & p0);
	PNode createChild(const Place & place, PNode widget);

	virtual Container * getContainer() = 0;
	PGtkContainer getObject();

protected:
	void initInstance();

	TypeId getChildType() { return childType; }
	void setChildType(const std::string & tname);
	bool isAsContainer() { return asContainer; }
	void setAsContainer(bool value) { asContainer= value; }
	bool placeholdersSet();

	void setAsContainerProperty(Property * property, Any value);
	Any getAsContainerProperty(Property * property);
	virtual void setChildren(Property * property, Any value) = 0;

	void setupAsContainer(bool value);
	void setupAsContainer();
	virtual void setupContainer() { }
	virtual void onAsContainerSetup() { }

	virtual Container * createContainer() = 0;
	PGtkWidget getWidget(PNode node);
	Children findChildren(const Point & p, int after);
	Children makeChildren(Any value);
	Children getChildren();

private:
	bool asContainer;
	TypeId childType;

};

template <class TContainer> class TGtkContainerView : public virtual GtkContainerView {

public:
	typedef typename TContainer::child_type TChild;
	typedef Glib::RefPtr<typename TContainer::container_type> PContainerType;

	PContainerType getObject()
		{ return PContainerType::cast_static(GlibObjectView::getObject()); }

protected:
	TGtkContainerView() { container= NULL; }

	~TGtkContainerView() {
		if(container)
			delete container;
	}

	TContainer * getContainer() {
		if(!container)
			container= createContainer();
		container->setContainer(GetPtr(getObject()));
		return container;
	}

	TContainer * createContainer() { return new TContainer(); }

private:
	TContainer * container;

};

class ChildView : public virtual GlibObjectView {

public:
	ChildView();

	PChild getObject();

protected:
	Any getWidget(Property * property);
	void setWidget(Property * property, Any value);

	Property & addIndexProperty();
	Any getIndex(Property * property);
	void setIndex(Property * property, Any value);

};

template <class TChild> class TChildView : public ChildView, public TCreateInstance<TChild> {

public:
	using TCreateInstance<TChild>::getObject;

};

}

#endif

==> crow-2.11.3.tar.bz2: crow-2.11.3/crow/views.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2008 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person 
 *  obtaining a copy of this software and associated documentation 
 *  files (the "Software"), to deal in the Software without 
 *  restriction, including without limitation the rights to use, copy, 
 *  modify, merge, publish, distribute, sublicense, and/or sell copies 
 *  of the Software, and to permit persons to whom the Software is 
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind, 
 *  express or implied, including but not limited to the warranties of 
 *  merchantability, fitness for a particular purpose and 
 *  noninfringement. In no event shall the authors or copyright 
 *  holders be liable for any claim, damages or other liability, 
 *  whether in an action of contract, tort or otherwise, arising from, 
 *  out of or in connection with the software or the use or other 
 *  dealings in the software.
 */

#ifndef CROW_VIEWS_H
#define CROW_VIEWS_H

#include "util.h"
#include "value.h"
#include "containers.h"
#include "crow_impl.h"

namespace Crow {

typedef sigc::slot<Any, Property *> PropertyValueGet;
typedef sigc::slot<void, Property *, Any> PropertyValueSet;
typedef sigc::slot<void, Property *, PEntityView> PropertyViewConfigure;
typedef sigc::slot<PNode, Property *, int> PropertyElementInsert;
typedef sigc::slot<std::string, Property *, PNode> PropertyLabelGet;

enum PropertyRole { prNone, prScalar, prVector, prEntity };

class Property {

public:
	Property();

	Property(
		const std::string & name,
		PropertyRole role,
		TypeId type
	);

	const std::string * getName() { return &name; }
	PropertyRole getRole() { return role; }
	TypeId getType() { return type; }

	Any getValue();
	void setValue(Any value);
	void configureView(PEntityView view);
	PNode insertElement(int index);
	std::string getLabel(PNode node);

	Property & setValueGet(const PropertyValueGet & get) { valueGet= get; return *this; }
	Property & setValueSet(const PropertyValueSet & set) { valueSet= set; return *this; }
	Property & setViewConfigure(const PropertyViewConfigure & configure) { viewConfigure= configure; return *this; }
	Property & setElementInsert(const PropertyElementInsert & insert) { elementInsert= insert; return *this; }
	Property & setLabelGet(const PropertyLabelGet & get) { labelGet= get; return *this; }

	StateFlags getState() { return state; }
	StateFlags getNodeState() { return state & sfNodeStateMask; }
	Property & setFlag(StateFlags sf) { Addition(&state, sf); return *this; }
	Property & clearFlag(StateFlags sf) { Subtraction(&state, sf); return *this; }
	bool hasFlags(StateFlags sf) { return Includes(state, sf); }
	bool isHidden() { return hasFlags(sfHidden); }
	bool isEditingAllowed() { return !hasFlags(sfNoEditing); }
	bool isReadOnly() { return hasFlags(sfReadOnly); }
	bool canEdit();

	EditorId getEditor() { return editor; }
	Property & setEditor(const std::string & ename);
	EditorId getElementEditor() { return elementEditor; }
	Property & setElementEditor(const std::string & ename);
	Any getDefault();
	Property & setDefault(Any dvalue);
	Any getInert();
	Property & setInert(Any value);
	Property & setInertDefault(Any value);

private:
	std::string name;
	PropertyRole role;
	TypeId type;
	PropertyValueGet valueGet;
	PropertyValueSet valueSet;
	PropertyViewConfigure viewConfigure;
	PropertyElementInsert elementInsert;
	PropertyLabelGet labelGet;
	StateFlags state;
	EditorId editor;
	EditorId elementEditor;
	Any defaultValue;
	Any inert;

};

class EntityView : public Object {

public:
	EntityView();

	void prepare(TypeId tid);
	void init(PNode node, Controller * controller, Any value=Any());
	void done();

	TypeId getType() { return type; }
	GType getGType();
	PNode getModelNode();

	Any getValue();

	bool isConfigured() { return configured; }
	void setConfigured();

	void update();
	void write();

	virtual bool isActiveWidget(Gtk::Widget * widget) { return true; }
	Property * findProperty(const std::string & pname);

protected:
	typedef std::list<Property> Properties;

	Properties * getProperties() { return &properties; }
	Controller * getController() { return controller; }
	Model * getModel();

	bool isEditing();
	bool isSeeking();
	bool isRoot();
	bool isInternal() { return internal; }

	void setValue(Any value);
	void clearValue();

	virtual Any createInstance();
	virtual void initInstance() { }
	virtual void doneInstance() { }

	void initDefaultValues();
	virtual void initDesign() { }

	Property & addProperty(
		const std::string & name,
		PropertyRole role,
		const std::string & type,
		Any dvalue=Any()
	);

	Property & addInertProperty(
		const std::string & name,
		PropertyRole role,
		const std::string & type,
		Any dvalue=Any()
	);

	Any getPropertyValueInert(Property * property);
	void setPropertyValueInert(Property * property, Any value);

	void moveBefore(const std::string & pname, const std::string & before);
	void touch(const std::string & str);
	void touch(PNode node, const std::string & str);
	Property * clearVectorProperty(const std::string & name);
	bool propertyModelVectorEmpty(const std::string & name);
	Any getPropertyModel(const std::string & name);

	template <class T> T getPropertyModel(const std::string & name, const T & defval) {
		Any value= getPropertyModel(name);
		return value ? *value->get<T>() : defval;
	}

private:
	TypeId type;
	PNode modelNode;
	Controller * controller;
	Any value;
	bool configured;
	bool internal;
	bool onceWritten;
	Properties properties;

};

class GlibObjectView : public EntityView {

public:
	GlibObjectView();

	Any getPropertyValue(Property * property);
	void setPropertyValue(Property * property, Any value);

	void setPropertyValueInertGet(Property * property, Any value);

	PNode insertObject(Property * property, int index);
	PNode insertEntity(Property * property, int index);
	PNode insertScalar(Property * property, int index, Any value);

	std::string findSignal(const std::string & name);

	PGlibObject getObject();
	template <class T> Glib::RefPtr<T> getObject() { return Glib::RefPtr<T>::cast_static(getObject()); }

protected:
	void doneInstance();

	void setReportDestruction(bool rd) { reportDestruction= rd; }

	Property & addProperty(
		const std::string & name,
		PropertyRole role,
		const std::string & type,
		Any dvalue=Any()
	);

	Property & addInertGetProperty(
		const std::string & name,
		PropertyRole role,
		const std::string & type,
		Any dvalue=Any()
	);

	PNode insertSignal(Property * property, int index);
	std::string getSignalLabel(Property * property, PNode node);

	Property & addSignalsProperty();

private:
	bool reportDestruction;

};

template <class T> class TCreateInstance : public virtual GlibObjectView {

public:
	Glib::RefPtr<T> getObject() { return GlibObjectView::getObject<T>(); }

protected:
	Any createInstance() { return CAny::createObject(getType(), NewRefPtr<T>()); }

};

struct FreePlace {
	Place place;
	Rectangle area;
};

typedef std::vector<FreePlace> FreePlaces;

class RelationEntry : public Glib::Object {

public:
	RelationEntry();

};

typedef Glib::RefPtr<RelationEntry> PRelationEntry;

class RelationEntryView : public TCreateInstance<RelationEntry> {

public:
	RelationEntryView();

};

class GtkWidgetView : public virtual GlibObjectView {

public:
	GtkWidgetView();

	virtual bool findFreePlace(const Point & p, FreePlace * fplace) { return false; }
	virtual FreePlaces findFreePlaces(const Point & p, const Nodes & nodes, const Point & p0) { return FreePlaces(); }
	virtual PNode createChild(const Place & place, PNode widget);

	virtual bool isTopChild(PChild child) { return true; }
	virtual Rectangle getChildArea();
	bool isInteractive() { return interactive; }

	PGtkWidget getObject();

protected:
	void initInstance();
	Any getSizeRequest(Property * property);
	void setSizeRequest(Property * property, Any value);

	Glib::ustring getDesignLabel();

	void addReceivesDefaultProperty(bool defval);
	void addCanFocusProperty(bool defval);
	void addCanDefaultProperty(bool defval);
	void addAppPaintableProperty();
	void addEventProperties();

	Point transform(const Point & p, PGtkWidget widget);
	Rectangle getWidgetRect(PGtkWidget widget);
	void setInteractive(bool b) { interactive= b; }

	std::string getRelationLabel(Property * property, PNode node);
	
private:
	bool interactive;

};

class GtkContainerViewBase : public virtual GtkWidgetView {

public:
	GtkContainerViewBase();

};

class GtkContainerView : public GtkContainerViewBase {

public:
	GtkContainerView();

	bool findFreePlace(const Point & p, FreePlace * fplace);
	FreePlaces findFreePlaces(const Point & p, const Nodes & nodes, const Point & p0);
	PNode createChild(const Place & place, PNode widget);

	virtual Container * getContainer() = 0;
	PGtkContainer getObject();

protected:
	void initInstance();

	TypeId getChildType() { return childType; }
	void setChildType(const std::string & tname);
	bool isAsContainer() { return asContainer; }
	void setAsContainer(bool value) { asContainer= value; }
	bool placeholdersSet();

	void setAsContainerProperty(Property * property, Any value);
	Any getAsContainerProperty(Property * property);
	virtual void setChildren(Property * property, Any value) = 0;

	void setupAsContainer(bool value);
	void setupAsContainer();
	virtual void setupContainer() { }
	virtual void onAsContainerSetup() { }

	virtual Container * createContainer() = 0;
	PGtkWidget getWidget(PNode node);
	Children findChildren(const Point & p, int after);
	Children makeChildren(Any value);
	Children getChildren();

private:
	bool asContainer;
	TypeId childType;

};

template <class TContainer> class TGtkContainerView : public virtual GtkContainerView {

public:
	typedef typename TContainer::child_type TChild;
	typedef Glib::RefPtr<typename TContainer::container_type> PContainerType;

	PContainerType getObject()
		{ return PContainerType::cast_static(GlibObjectView::getObject()); }

protected:
	TGtkContainerView() { container= NULL; }

	~TGtkContainerView() {
		if(container)
			delete container;
	}

	TContainer * getContainer() {
		if(!container)
			container= createContainer();
		container->setContainer(GetPtr(getObject()));
		return container;
	}

	TContainer * createContainer() { return new TContainer(); }

private:
	TContainer * container;

};

class ChildView : public virtual GlibObjectView {

public:
	ChildView();

	PChild getObject();

protected:
	Any getWidget(Property * property);
	void setWidget(Property * property, Any value);

	Property & addIndexProperty();
	Any getIndex(Property * property);
	void setIndex(Property * property, Any value);

};

template <class TChild> class TChildView : public ChildView, public TCreateInstance<TChild> {

public:
	using TCreateInstance<TChild>::getObject;

};

}

#endif
